/*
 MobaLedLib: LED library for model railways
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 Copyright (C) 2018, 2019  Hardi Stengelin: MobaLedLib@gmx.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 -------------------------------------------------------------------------------------------------------------


 MobaLedLib.h
 ~~~~~~~~~~~~

 This is the main headder file for the MobaLedLib.

*/
#ifndef _MOBALEDLIB_H_
#define _MOBALEDLIB_H_

#define MobaLedLib_Ver  1.0.1   // Adapt also library.properties if changed

#define FASTLED_INTERNAL        // Disable version number message in FastLED library (looks like an error)

#include "FastLED.h"            // The FastLED library must be installed in addition if you got the
                                // error message "..fatal error: FastLED.h: No such file or directory"
                                // Arduino IDE: Sketch / Include library / Manage libraries                    Deutsche IDE: Sketch / Bibliothek einbinden / Bibliothek verwalten
                                //              Type "FastLED" in the "Filter your search..." field                          "FastLED" in das "Grenzen Sie ihre Suche ein" Feld eingeben
                                //              Select the entry and click "Install"                                         Gefundenen Eintrag auswaehlen und "Install" anklicken


#include "Dprintf.h"            // Debug Ausgaben

#include "Lib_Config.h"         // Library configurarion
#include "IntDefs.h"            // Internal defines

#include "LED_HeartBeat.h"


// Error numbers which are generated by the compiler: "warning: narrowing conversion of 'xxx' from 'int' to 'const unsigned char' inside { } [-Wnarrowing]
#define _ERR_INPUT_RANGE    256 // Is generated if the In number is >= 256       (_INP_STRUCT_ARRAY_SIZE * 4)
#define _ERR_LED_RANGE      512 // Is generated if the LED number is >= NUM_LEDS


#define _INP_STRUCT_ARRAY_SIZE  64       // _INP_STRUCT_ARRAY_SIZE*4 defines the maximal number of inputs (Maximal 64)

#define _ChkIn(In) ((In) < _INP_STRUCT_ARRAY_SIZE*4 ? (In) : _ERR_INPUT_RANGE) // Generate a compiler warning if the input channel is invalid
                                                                               // In this case th input is mapped to 0.
                                                                               //   LEDs_Eisenbahn.ino:268:3: warning: narrowing conversion of '256' from 'int' to 'const unsigned char' inside { } [-Wnarrowing]
                                                                               //   LEDs_Eisenbahn.ino:268:3: warning: large integer implicitly truncated to unsigned type [-Woverflow]
                                                                               // Normaly this message is not generated because _INP_STRUCT_ARRAY_SIZE is set to 64 => 256 inputs are possible

// Todo: Warum wird hier uint32_t und nicht uint16_t verwendet ?
#define _W2B(w)   (((uint32_t)(w))&0xFF), (((uint32_t)(w))>>8)             // Split a 16 bit number into two bytes. Maximal value 65535
#define _T2B(t)   _W2B(t)                                                  // Split a time into two bytes. Maximal value 65535ms = 65.5 Sec.


// Automatisches umschalten der Zeiten fuer RandMux() und Random()
// Wenn die Zeiten groesser als 65535 werden wird automatisch die Zeitbasis von ms auf 16 ms umgeschaltet.
// Dazu wird der Mode RF_SLOW gesetzt und die Zeiten durch 16 geteilt.
// Die Logik dazu war verzwickt weil die UND Verknuepfung "&&" als Argument des _T2B() Makros nicht funktioniert hat.
// Vielleicht lag es gar nicht an der UND Verknuepfung sondern daran, das um das _T2B() Makro noch eine Klammer war.
// Darum wurde die UND Funktion ueber eine Multiplikation geloest. Das Makro _LT_MX() liefert eine 1 wenn das Argument
// kleiner als _MAX_TIMER_VAL ist. Mehere _LT_MX() Ausdruecke koennen miteinander multipliziert werden. Das Ergebnis
// ist nur dann 1 wenn alle Teile zutreffen.

// Disable warning in _ALL_LT_MX2 and _ALL_LT_MX4
#define GCC_VERSION (__GNUC__ * 10000L + __GNUC_MINOR__ * 100L + __GNUC_PATCHLEVEL__)
#if GCC_VERSION >= 70300L     // Don't know exactly in which version this check was added                     // 07.10.19:
  #pragma GCC diagnostic ignored "-Wint-in-bool-context"                                                      // 05.10.19:
#endif
// Todo: Noch mal mit logischer verknuepfung und nit mit "*" versuchen. Dann kann das #pragma wieder entfallen

#define _MAX_TIMER_VAL 65536  // Could be chaged to test the automatic switching
#define _LT_MX(MaxTime)                                     (((MaxTime)<_MAX_TIMER_VAL)?1:0)
#define _ALL_LT_MX2(MinTime,MaxTime)                        ((_LT_MX(MinTime) * _LT_MX(MaxTime)  * _LT_MX((MinTime)+(MaxTime))))  // Die Multiplikation kann wie eine UND Verknuepfung verwendet werden
#define _ALL_LT_MX4(MinTime1, MaxTime1, MinTime2, MaxTime2) (_ALL_LT_MX2((MinTime1), (MaxTime1)) * _ALL_LT_MX2((MinTime2), (MaxTime2)))
#define _RAND_MODE2(MinTime,  MaxTime)                      (_ALL_LT_MX2((MinTime ), (MaxTime )                    )?0:RF_SLOW)
#define _RAND_MODE4(MinTime1, MaxTime1, MinTime2, MaxTime2) (_ALL_LT_MX4((MinTime1), (MaxTime1), MinTime2, MaxTime2)?0:RF_SLOW)
#define _RAND_DIV2( MinTime,  MaxTime)                      (_ALL_LT_MX2((MinTime), (MaxTime)                          )?1L:16L)
#define _RAND_DIV4( MinTime1, MaxTime1, MinTime2, MaxTime2) (_ALL_LT_MX4((MinTime1),(MaxTime1), (MinTime2), (MaxTime2) )?1L:16L)
#define _Tx2B2(Time, OtherTime)                             _T2B((Time)/_RAND_DIV2((Time), (OtherTime)))                            // Hier darf aussen herum kleine Klammer hinzugefuegt werden weil das Ergebnis zwei durch Komma getrennte Bytes sind
#define _Tx2B4(Time, OtherTime1, OtherTime2, OtherTime3)    _T2B((Time)/_RAND_DIV4((Time), (OtherTime1), (OtherTime2), (OtherTime3)))

// Checking the leds[] array size. Therefore "NUM_LEDS" must be defined in the sketch.
#define _CHKL(LED) ((LED) < NUM_LEDS ? (LED) : _ERR_LED_RANGE) // If you got the error "'NUM_LEDS' was not declared in this scope" you have do add a #define NUM_LEDS ... line to your sketch
                                                               // The warnung "warning: narrowing conversion of '512' from 'int' to 'const unsigned char' inside { } [-Wnarrowing]"
                                                               // is generated if the LED number is >= NUM_LEDS

// 13.09.19:  Incremented the number of Pattern macros to 30 (Old 20)

// Maximal time for one time interval 65.535 Sec
#define PatternT1( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            PATTERNT1_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT2( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          PATTERNT2_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT3( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       PATTERNT3_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT4( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    PATTERNT4_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT5( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 PATTERNT5_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT6( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              PATTERNT6_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT7( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           PATTERNT7_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT8( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        PATTERNT8_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT9( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     PATTERNT9_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT10(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 PATTERNT10_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT11(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             PATTERNT11_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT12(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         PATTERNT12_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT13(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     PATTERNT13_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT14(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 PATTERNT14_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT15(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             PATTERNT15_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT16(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         PATTERNT16_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT17(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     PATTERNT17_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT18(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 PATTERNT18_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT19(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             PATTERNT19_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT20(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         PATTERNT20_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT21(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     PATTERNT21_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT22(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 PATTERNT22_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT23(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             PATTERNT23_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT24(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         PATTERNT24_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT25(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     PATTERNT25_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT26(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 PATTERNT26_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT27(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             PATTERNT27_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT28(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         PATTERNT28_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT29(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     PATTERNT29_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternT30(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) PATTERNT30_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,

// Es werden zwei verschidene Makro Gruppen benoetigt weil
// die digitale Version 5 Byte RAM benoetigt und die Analoge 7 Bytes.

#define APatternT1( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            APATTERNT1_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT2( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          APATTERNT2_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT3( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       APATTERNT3_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT4( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    APATTERNT4_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT5( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 APATTERNT5_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT6( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              APATTERNT6_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT7( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           APATTERNT7_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT8( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        APATTERNT8_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT9( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     APATTERNT9_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT10(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 APATTERNT10_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT11(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             APATTERNT11_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT12(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         APATTERNT12_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT13(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     APATTERNT13_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT14(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 APATTERNT14_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT15(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             APATTERNT15_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT16(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         APATTERNT16_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT17(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     APATTERNT17_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT18(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 APATTERNT18_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT19(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             APATTERNT19_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT20(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         APATTERNT20_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT21(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     APATTERNT21_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT22(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 APATTERNT22_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT23(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             APATTERNT23_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT24(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         APATTERNT24_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT25(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     APATTERNT25_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT26(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 APATTERNT26_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT27(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             APATTERNT27_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT28(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         APATTERNT28_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT29(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     APATTERNT29_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternT30(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) APATTERNT30_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,

/*
Mit USE_XFADE:
Der Sketch verwendet 20032 Bytes (65%) des Programmspeicherplatzes. Das Maximum sind 30720 Bytes.
Globale Variablen verwenden 786 Bytes (38%) des dynamischen Speichers, 1262 Bytes fuer lokale Variablen verbleiben. Das Maximum sind 2048 Bytes.

Ohne:
Der Sketch verwendet 19812 Bytes (64%) des Programmspeicherplatzes. Das Maximum sind 30720 Bytes.
Globale Variablen verwenden 786 Bytes (38%) des dynamischen Speichers, 1262 Bytes fuer lokale Variablen verbleiben. Das Maximum sind 2048 Bytes.
                       220
*/
#define USE_XFADE // 220 Bytes Flash
#ifdef USE_XFADE
// Drittes Makro bei dem fuer jede LED ein Byte RAM Reserviert wird. Das wird benoetigt wenn das flag _PF_XFADE gesetzt ist.
// Dummerweise kann bei dieser Funktion keine Berechnung beim Parameter LED gebacht werden ;-(
 #define XPatternT1( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            APATTERNT1_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT2( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          APATTERNT2_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT3( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       APATTERNT3_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT4( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    APATTERNT4_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT5( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 APATTERNT5_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT6( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              APATTERNT6_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT7( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           APATTERNT7_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT8( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        APATTERNT8_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT9( LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     APATTERNT9_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT10(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 APATTERNT10_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT11(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             APATTERNT11_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT12(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         APATTERNT12_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT13(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     APATTERNT13_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT14(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 APATTERNT14_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT15(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             APATTERNT15_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT16(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         APATTERNT16_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT17(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     APATTERNT17_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT18(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 APATTERNT18_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT19(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             APATTERNT19_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT20(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         APATTERNT20_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT21(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     APATTERNT21_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT22(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 APATTERNT22_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT23(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             APATTERNT23_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT24(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         APATTERNT24_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT25(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     APATTERNT25_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT26(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 APATTERNT26_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT27(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             APATTERNT27_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT28(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         APATTERNT28_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT29(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     APATTERNT29_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternT30(LED,NStru,InCh,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) APATTERNT30_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),SI_1,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #endif

// Same macros with an enable input 05.11.18:

#define PatternTE1( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            PATTERNT1_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE2( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          PATTERNT2_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE3( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       PATTERNT3_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE4( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    PATTERNT4_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE5( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 PATTERNT5_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE6( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              PATTERNT6_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE7( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           PATTERNT7_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE8( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        PATTERNT8_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE9( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     PATTERNT9_T, _CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE10(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 PATTERNT10_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE11(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             PATTERNT11_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE12(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         PATTERNT12_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE13(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     PATTERNT13_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE14(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 PATTERNT14_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE15(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             PATTERNT15_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE16(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         PATTERNT16_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE17(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     PATTERNT17_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE18(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 PATTERNT18_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE19(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             PATTERNT19_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE20(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         PATTERNT20_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE21(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     PATTERNT21_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE22(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 PATTERNT22_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE23(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             PATTERNT23_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE24(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         PATTERNT24_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE25(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     PATTERNT25_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE26(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 PATTERNT26_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE27(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             PATTERNT27_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE28(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         PATTERNT28_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE29(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     PATTERNT29_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define PatternTE30(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) PATTERNT30_T,_CHKL(LED)+RAM5,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,

#define APatternTE1( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            APATTERNT1_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE2( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          APATTERNT2_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE3( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       APATTERNT3_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE4( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    APATTERNT4_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE5( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 APATTERNT5_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE6( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              APATTERNT6_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE7( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           APATTERNT7_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE8( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        APATTERNT8_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE9( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     APATTERNT9_T, _CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE10(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 APATTERNT10_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE11(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             APATTERNT11_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE12(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         APATTERNT12_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE13(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     APATTERNT13_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE14(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 APATTERNT14_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE15(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             APATTERNT15_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE16(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         APATTERNT16_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE17(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     APATTERNT17_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE18(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 APATTERNT18_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE19(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             APATTERNT19_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE20(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         APATTERNT20_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE21(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     APATTERNT21_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE22(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 APATTERNT22_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE23(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             APATTERNT23_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE24(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         APATTERNT24_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE25(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     APATTERNT25_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE26(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 APATTERNT26_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE27(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             APATTERNT27_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE28(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         APATTERNT28_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE29(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     APATTERNT29_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#define APatternTE30(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) APATTERNT30_T,_CHKL(LED)+RAM7,(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,

#ifdef USE_XFADE
// Drittes Makro bei dem fuer jede LED ein Byte RAM Reserviert wird. Das wird benoetigt wenn das flag _PF_XFADE gesetzt ist.
// Dummerweise kann bei dieser Funktion keine Berechnung beim Parameter LED gebacht werden ;-(
 #define XPatternTE1( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1, ...)                                                                                                            APATTERNT1_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE2( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,...)                                                                                                          APATTERNT2_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE3( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,...)                                                                                                       APATTERNT3_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE4( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,...)                                                                                                    APATTERNT4_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE5( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,...)                                                                                                 APATTERNT5_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE6( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,...)                                                                                              APATTERNT6_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE7( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,...)                                                                                           APATTERNT7_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE8( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,...)                                                                                        APATTERNT8_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE9( LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,...)                                                                                     APATTERNT9_T, _CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE10(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,...)                                                                                 APATTERNT10_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE11(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,...)                                                                             APATTERNT11_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE12(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,...)                                                                         APATTERNT12_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE13(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,...)                                                                     APATTERNT13_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE14(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,...)                                                                 APATTERNT14_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE15(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,...)                                                             APATTERNT15_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE16(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,...)                                                         APATTERNT16_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE17(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,...)                                                     APATTERNT17_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE18(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,...)                                                 APATTERNT18_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE19(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,...)                                             APATTERNT19_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE20(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,...)                                         APATTERNT20_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode|_PF_XFADE,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE21(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,...)                                     APATTERNT21_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE22(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,...)                                 APATTERNT22_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE23(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,...)                             APATTERNT23_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE24(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,t24,...)                         APATTERNT24_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE25(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,...)                     APATTERNT25_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE26(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,...)                 APATTERNT26_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE27(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,...)             APATTERNT27_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE28(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,...)         APATTERNT28_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE29(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,...)     APATTERNT29_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
 #define XPatternTE30(LED,NStru,InCh,Enable,LEDs,Val0,Val1,Off,Mode,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22,T23,T24,T25,T26,T27,T28,T29,T30,...) APATTERNT30_T,_CHKL(LED)+RAM7+RAMN(LEDs),(NStru)&0xFF,_ChkIn(InCh),Enable,LEDs,Val0,Val1,Off,Mode,_T2B(T1),_T2B(T2),_T2B(T3),_T2B(T4),_T2B(T5),_T2B(T6),_T2B(T7),_T2B(T8),_T2B(T9),_T2B(T10),_T2B(T11),_T2B(T12),_T2B(T13),_T2B(T14),_T2B(T15),_T2B(T16),_T2B(T17),_T2B(T18),_T2B(T19),_T2B(T20),_T2B(T21),_T2B(T22),_T2B(T23),_T2B(T24),_T2B(T25),_T2B(T26),_T2B(T27),_T2B(T28),_T2B(T29),_T2B(T30),_W2B(COUNT_VARARGS(__VA_ARGS__)), __VA_ARGS__,
#endif

#ifdef _HOUSE_STATISTIC
  #define RAMH RAM12     // Additional RAM for statistics: sizeof(House_Stat_T)
#else
  #define RAMH RAM2
#endif

/*
 Wenn die Const() Funktion ueber die Pattern Funktion implementiert werden,
 dann benoetigt das 13 Byte pro Konstante und 5 Bytes RAM.
 Die Proc_Const() Funktion benoetigt 6 Byte pro Zeile und 188 Bytes C-Code
 => Aus der Sicht des FLASH lohnt es die Proc_Const() Funktion zu verwenden
   wenn mehr als 26 Const() Zeilen verwendet werden.
 Man koennte die Pattern Funktion so erweitern, dass sie keinen RAM und keine Zeiten benutzt
 Das wurde 2 Bytes FLASH sparen
*/



#ifdef _USE_SEP_CONST
 #define Const(         LED,Cx,InCh,Val0, Val1)               CONST_T, _CHKL(LED)+RAM0,Cx,_ChkIn(InCh),Val0, Val1,               // 6 Bytes / 13 Bytes mit der Pattern Funktion + 5 Bytes RAM
#else
 #define Const(         LED,Cx,InCh,Val0, Val1)               PatternT1(LED,_NStru(Cx,   1,1),InCh,_Cx2LedCnt(Cx),Val0,Val1,Val0,0,0 Sec, _Cx2P_BLINK(Cx))
#endif



#define  House(         LED,InCh, On_Min,On_Limit, ...)                     HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),On_Min,     On_Limit,HOUSE_MIN_T,HOUSE_MAX_T,COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Room types
#define  HouseT(        LED,InCh, On_Min,On_Limit,Min_T,Max_T,...)          HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),On_Min,     On_Limit,Min_T,      Max_T,      COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Room types
#define  GasLights(     LED,InCh, ...)                                      HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),127,        255,     GAS_MIN_T,  GAS_MAX_T,  COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Gas lights
#define  House_Inv(     LED,InCh, On_Min,On_Limit, ...)                     HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),On_Min|0x80,On_Limit,HOUSE_MIN_T,HOUSE_MAX_T,COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Room types
#define  HouseT_Inv(    LED,InCh, On_Min,On_Limit,Min_T,Max_T,...)          HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),On_Min|0x80,On_Limit,Min_T,      Max_T,      COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Room types
#define  GasLights_Inv( LED,InCh, ...)                                      HOUSE_T,     _CHKL(LED)+RAMH,_ChkIn(InCh),127   |0x80,255,     GAS_MIN_T,  GAS_MAX_T,  COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,  // Variable number of Gas lights
#define  Fire(          LED,InCh, LedCnt, Brightness)                       FIRE_T,      _CHKL(LED)+RAM1+RAMN(LedCnt),_ChkIn(InCh),LedCnt,Brightness,
#ifdef _NEW_ROOM_COL
 #define Set_ColTab(    r0,g0,b0,r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4,r5,g5,b5,r6,g6,b6,r7,g7,b7,r8,g8,b8,r9,g9,b9,r10,g10,b10,r11,g11,b11,r12,g12,b12,r13,g13,b13,r14,g14,b14,r15,g15,b15,r16,g16,b16) SET_COLTAB_T, r0,g0,b0,r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4,r5,g5,b5,r6,g6,b6,r7,g7,b7,r8,g8,b8,r9,g9,b9,r10,g10,b10,r11,g11,b11,r12,g12,b12,r13,g13,b13,r14,g14,b14,r15,g15,b15,r16,g16,b16,
#endif
#if _USE_SET_TVTAB                                                                                            // 10.01.20:
  #define Set_TV_COL1(  InCh, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R_Min, R_Max, G_Min, G_Max, B_Min, B_Max)   SET_TV_TAB_T, InCh, 0x00, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R_Min, R_Max, G_Min, G_Max, B_Min, B_Max,
  #define Set_TV_COL2(  InCh, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R_Min, R_Max, G_Min, G_Max, B_Min, B_Max)   SET_TV_TAB_T, InCh, 0x01, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R_Min, R_Max, G_Min, G_Max, B_Min, B_Max,
  #define Set_TV_BW1(   InCh, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R,            G,            B)              SET_TV_TAB_T, InCh, 0x00, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R,     R,     G,     G,     B,     B,
  #define Set_TV_BW2(   InCh, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R,            G,            B)              SET_TV_TAB_T, InCh, 0x01, Update_t_Min, Update_t_Max, Min_Brightness, Max_Brightness, R,     R,     G,     G,     B,     B,
#endif

#if _USE_DEF_NEON                                                                                             // 12.01.20:
  #define Set_Def_Neon( InCh, Rand_On, RandOff, Min_Def)                    SET_DEF_NEON_T, InCh, Rand_On, RandOff, Min_Def,
#endif

#define _CHECK_MODE_RF_NOT_SAME(Mode, DstVar1, DstVarN) (((Mode)&RF_NOT_SAME) && (DstVarN) - (DstVar1) == 0) ? ((Mode) & ~RF_NOT_SAME) : (Mode) // Prevent endless loop if DstVar1 == DstVarN     // 13.01.20:

#define  Logic(         DstVar, ...)                                        LOGIC_T,     DstVar, COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,
#define  Counter(       Mode, InCh, Enable, TimeOut, ...)                   COUNTER_T,   _W2B(Mode), InCh+RAM3, Enable, _T2B((TimeOut)/16), COUNT_VARARGS(__VA_ARGS__), __VA_ARGS__,      // TimeOut maximal 17 Min
#define  RandMux(       DstVar1, DstVarN, InCh, Mode, MinTime, MaxTime)     RANDMUX_T,   DstVar1+RAM4, DstVarN, InCh, _CHECK_MODE_RF_NOT_SAME(Mode, DstVar1, DstVarN) | _RAND_MODE2((MinTime), (MaxTime)),                   _Tx2B2(MinTime, MaxTime),               _Tx2B2(MaxTime, MinTime),
#define  RandCntMux(    DstVar1, DstVarN, InCh, Mode, MinTime, MaxTime)     RandMux(DstVar1, DstVarN, InCh, Mode|RF_SEQ, MinTime, MaxTime)
#define  Random(        DstVar, InCh, Mode, MinTime, MaxTime, MinOn, MaxOn) RANDOM_T,    DstVar +RAM4,          InCh, (Mode) | _RAND_MODE4((MinTime), (MaxTime), (MinOn), (MaxOn)), _Tx2B4(MinTime, MaxTime, MinOn, MaxOn), _Tx2B4(MaxTime, MinTime, MinOn, MaxOn), _Tx2B4(MinOn, MaxOn, MinTime, MaxTime), _Tx2B4(MaxOn, MinOn, MinTime, MaxTime),
#define  Welding(       LED, InCh)                                          WELDING_T,   _CHKL(LED)+RAM5, InCh,
#define  RandWelding(   LED, InCh, Var, MinTime, MaxTime, MinOn, MaxOn)     Random(Var, InCh, RM_NORMAL, MinTime, MaxTime, MinOn, MaxOn) Welding(LED, Var)



#define  CopyLED(       LED, InCh, SrcLED)                                  COPYLED_T,   _CHKL(LED), InCh, SrcLED,
#define  Schedule(      DstVar1, DstVarN, EnableCh, Start, End)             SCHEDULE_T,  DstVar1+RAM3, DstVarN, EnableCh, Start, End,  // Zeit- oder Helligkeitsgesteuertes Ein- und Ausschalten von Variablen.
#define  New_HSV_Group()                                                    NEW_HSV_GROUP_T+RAM3,
#define  New_Local_Var()                                                    NEW_LOCAL_VAR_T+RAM2,             // 07.11.18:
#define  Use_GlobalVar( GlobVarNr)                                          USE_GLOBALVAR_T, GlobVarNr,
#define  InCh_to_TmpVar(FirstInCh, InCh_Cnt)                                INCH_TO_TMPVAR_T, FirstInCh, InCh_Cnt,
#define  Bin_InCh_to_TmpVar(FirstInCh, InCh_Cnt)                            BIN_INCH_TO_TMPVAR_T, FirstInCh, InCh_Cnt, // 18.01.19:

#define  Button(        LED,Cx,InCh,Duration,Val0, Val1)      PatternT1(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx),Val0,Val1,Val0,PM_SEQUENZ_W_ABORT+PF_SLOW,Duration/16,_Cx2P_BLINK(Cx))
#define  ButtonNOff(    LED,Cx,InCh,Duration,Val0, Val1)      PatternT1(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx),Val0,Val1,Val0,PM_SEQUENZ_NO_RESTART+PF_SLOW,Duration/16,_Cx2P_BLINK(Cx)) // 12.03.19:
#define  Blinker(       LED,Cx,InCh,Period)                   PatternT1(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx), 0,  255, 0,0,(Period)/2,                    _Cx2P_BLINK(Cx))     // Blinker
#define  BlinkerInvInp( LED,Cx,InCh,Period)                   PatternT1(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx), 0,  255, 0,PF_INVERT_INP,(Period)/2,        _Cx2P_BLINK(Cx))     // Blinker with inverse input
#define  BlinkerHD(     LED,Cx,InCh,Period)                   PatternT1(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx),50,  255, 0,0,(Period)/2,                    _Cx2P_BLINK(Cx))     // Hell/Dunkel Blinken
#define  Blink2(        LED,Cx,InCh,Pause,Act,Val0,Val1)      PatternT2(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx),Val0,Val1,0,0,Act,     Pause,                _Cx2P_BLINK(Cx))     // Blinken mit zwei verschiedenen Helligkeiten
#define  Blink3(        LED,Cx,InCh,Pause,Act,Val0,Val1,Off)  PatternT2(LED,_NStru(Cx,   2,1),InCh,_Cx2LedCnt(Cx),Val0,Val1,Off,0,Act,   Pause,                _Cx2P_BLINK(Cx))     // Blinken mit zwei verschiedenen Helligkeiten und einen Wert wenn der Blinker aus ist
#define  BlueLight1(    LED,Cx,InCh)                          PatternT4(LED,_NStru(Cx,   4,1),InCh,_Cx2LedCnt(Cx),0,   255, 0,0,  48 ms, 74 ms,  48 ms, 496 ms,_Cx2P_DBLFL(Cx))     // Blaulicht
#define  BlueLight2(    LED,Cx,InCh)                          PatternT4(LED,_NStru(Cx,   4,1),InCh,_Cx2LedCnt(Cx),0,   255, 0,0,  24 ms, 74 ms,  24 ms, 512 ms,_Cx2P_DBLFL(Cx))     // Blaulicht (Andere Periode damit nicht synchron)
#define  Leuchtfeuer(   LED,Cx,InCh)                          PatternT4(LED,_NStru(Cx,   4,1),InCh,_Cx2LedCnt(Cx),0,   255, 0,0,1000 ms,500 ms,1000 ms,1500 ms,_Cx2P_DBLFL(Cx))     // Leuchtfeuer fuer Windrad
#define  LeuchtfeuerALL(LED,InCh)                             PatternT4(LED,_NStru(C_ALL,4,1),InCh,_Cx2LedCnt(C_ALL),0,255, 0,0,1000 ms,500 ms,1000 ms,1500 ms,B11000111, B00000001)// Leuchtfeuer fuer Windrad
#define  Andreaskreuz(  LED,Cx,InCh)                         APatternT1(LED,_NStru(Cx,   0,1),InCh,2,0,255,0,0,250 ms,105)                                                          // Nur fuer einzelne LEDs (C1, C2, C3).
#define  AndreaskrRGB(  LED,InCh)                            APatternT1(LED,0,                InCh,4,0,255,0,0,250 ms,129,24)
#define  RGB_AmpelX(    LED,InCh)                             PatternT4(LED,0,InCh,18,0,255,0,0,3 Sec,1 Sec,10 Sec,3 Sec,1,2,100,8,0,40,0,134,0,1,2,4,200,16,0,80,0,12)             // Ampel fuer Kreuzung mit 6 RGB LEDs
#define  RGB_AmpelXFade(LED,InCh)                            APatternT8(LED,0,InCh,18,0,255,0,0,200,2 Sec,200,1 Sec,200,10 Sec,200,3 Sec,1,2,4,8,144,33,64,134,0,128,2,0,10,128,33,0,134,0,1,2,4,8,16,32,67,128,12,1,0,5,0,20,0,67,0,12)

#define  AmpelX(        LED,InCh)                             PatternT4(LED,0,InCh,6,0, 255,0,0,3 Sec,1 Sec,10 Sec,3 Sec,201,194,40,73,22,70)                                       // Ampel fuer Kreuzung mit 6 einzelnen LEDs (Achtung keine RGB LEDs)
#define  Flash(         LED, Cx, InCh, Var, MinTime, MaxTime) Random(Var, InCh, RM_NORMAL, MinTime, MaxTime, 30 ms, 30 ms) Const(LED, Cx, Var, 0, 255)                                         // Zufaelliges Blitzlicht (Fotograf)

#define  Def_Neon_Misha(LED, InCh, Var, Min_Delay, Max_Delay) Random(Var, InCh, RM_NORMAL, Min_Delay, Max_Delay, 500 ms, 2000 ms)  \
                                                              PatternT8(LED,8, Var,3,1,128,0,0,300 ms,100 ms,300 ms,100 ms,4 sec,400 ms,100 ms,100 ms,0,254,3,248,191,136,17,128,100)



#define ConstrWarnLightRGB6(   LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT12(LED,28,InCh,18,0,255,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MaxBrightness,MaxBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0,MinBrightness,MinBrightness,0)

#define ConstrWarnLightFlash3( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT6( LED,128,InCh,3, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitE,129,64,0,0,0)
#define ConstrWarnLightFlash4( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT8( LED,0,  InCh,4, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,2,4,8,0,0)
#define ConstrWarnLightFlash5( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT10(LED,128,InCh,5, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,8,64,0,2,16,0,0)
#define ConstrWarnLightFlash6( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT12(LED,0,  InCh,6, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,32,0,4,128,0,16,0,2)
#define ConstrWarnLightFlash7( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT14(LED,192,InCh,7, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,128,0,64,0,32,0,16,0,8,0,4,0)
#define ConstrWarnLightFlash8( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT16(LED,0,  InCh,8, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,2,0,4,0,8,0,16,0,32,0,64,0,128,0)
#define ConstrWarnLightFlash9( LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT18(LED,192,InCh,9, MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,8,0,64,0,0,2,0,16,0,128,0,0,4,0,32,0,0,1,0)
#define ConstrWarnLightFlash10(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT20(LED,0,  InCh,10,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,32,0,0,4,0,128,0,0,16,0,0,2,0,64,0,0,8,0,0,1,0,32,0)
#define ConstrWarnLightFlash11(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT22(LED,192,InCh,11,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,128,0,0,64,0,0,32,0,0,16,0,0,8,0,0,4,0,0,2,0,0,1,0,128,0,0,64,0,0)
#define ConstrWarnLightFlash12(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT24(LED,0,  InCh,12,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,0,2,0,0,4,0,0,8,0,0,16,0,0,32,0,0,64,0,0,128,0,0,0,1,0,0,2,0,0,4,0,0,8,0)
#define ConstrWarnLightFlash13(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT26(LED,192,InCh,13,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,0,8,0,0,64,0,0,0,2,0,0,16,0,0,128,0,0,0,4,0,0,32,0,0,0,1,0,0,8,0,0,64,0,0,0,2,0,0,16,0,0)
#define ConstrWarnLightFlash14(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT28(LED,0,  InCh,14,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,0,32,0,0,0,4,0,0,128,0,0,0,16,0,0,0,2,0,0,64,0,0,0,8,0,0,0,1,0,0,32,0,0,0,4,0,0,128,0,0,0,16,0,0,0,2,0)
#define ConstrWarnLightFlash15(LED,InCh, MinBrightness, MaxBrightness, OnT, WaitT, WaitE)      PatternT30(LED,192,InCh,15,MinBrightness,MaxBrightness,0,0,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitT,OnT,WaitE,1,0,0,128,0,0,0,64,0,0,0,32,0,0,0,16,0,0,0,8,0,0,0,4,0,0,0,2,0,0,0,1,0,0,128,0,0,0,64,0,0,0,32,0,0,0,16,0,0,0,8,0,0,0,4,0,0)

#define ConstrWarnLight3(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT4( LED,128,InCh,3, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,WaitE,17,1,0,0,0)
#define ConstrWarnLight4(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT5( LED,0,  InCh,4, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,WaitE,33,132,0,0,0,0)
#define ConstrWarnLight5(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT6( LED,128,InCh,5, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,WaitE,65,16,4,1,0,0,0,0)
#define ConstrWarnLight6(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT7( LED,0,  InCh,6, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,129,64,32,16,8,0,0,0,0)
#define ConstrWarnLight7(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT8( LED,128,InCh,7, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,1,1,1,1,1,1,0,0,0,0)
#define ConstrWarnLight8(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT9( LED,0,  InCh,8, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,2,4,8,16,32,64,128,0,0,0,0)
#define ConstrWarnLight9(      LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT10(LED,128,InCh,9, MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,4,16,64,0,1,4,16,64,0,1,0,0,0)
#define ConstrWarnLight10(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT11(LED,0,  InCh,10,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,8,64,0,2,16,128,0,4,32,0,1,8,0,0)
#define ConstrWarnLight11(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT12(LED,128,InCh,11,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,0)
#define ConstrWarnLight12(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT13(LED,128,InCh,12,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,32,0,4,128,0,16,0,2,64,0,8,0,1,32,0,4,128,0,0)
#define ConstrWarnLight13(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT14(LED,64, InCh,13,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,64,0,16,0,4,0,1,64,0,16,0,4,0,1,64,0,16,0,4,0,1,0)
#define ConstrWarnLight14(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT15(LED,192,InCh,14,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,128,0,64,0,32,0,16,0,8,0,4,0,2,0,1,128,0,64,0,32,0,16,0,8,0,0)
#define ConstrWarnLight15(     LED,InCh, MinBrightness, MaxBrightness, OnT, WaitE)             PatternT16(LED,0,  InCh,15,MinBrightness,MaxBrightness,0,0,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,OnT,WaitE,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0)

// Control the charlieplexing Modul
#define Charlie_Buttons(LED, InCh, States)    InCh_to_TmpVar(InCh, States) \
                                              PatternT1(LED,29,SI_LocalVar,2,0,255,0,0,1 ms,15,15,35,35,55,55,75,75,95,95,115,115,135,135,155,155,175,175,195,195,215,215  ,63,191,191,191,191,191,191,191,191,191,191)

#define _GET_BIN_SIZE(n)  (n<=1?0:(n<=2?1:(n<=4)?2:(n<=8?3:(n<=16?4:999)))) // Number of bits needed for n states (n up to 16)
#define Charlie_Binary(LED, InCh, BinStates)  Bin_InCh_to_TmpVar(InCh, _GET_BIN_SIZE((BinStates))) \
                                              PatternT1(LED,29,SI_LocalVar,2,0,255,0,0,1 ms,15,15,35,35,55,55,75,75,95,95,115,115,135,135,155,155,175,175,195,195,215,215  ,63,191,191,191,191,191,191,191,191,191,191)


// Entry signal with 3 aspects
#define EntrySignal3_RGB(LED, InCh)     InCh_to_TmpVar(InCh, 3)      /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,160,SI_LocalVar,9,0,128,0,0,125 ms,64,4,104,0  ,63,191,191)

#define EntrySignal3(LED, InCh)         InCh_to_TmpVar(InCh, 3)      /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,224,SI_LocalVar,3,0,128,0,0,125 ms,145,1  ,63,191,191)

// Departure signal with 4 aspects
#define DepSignal4_RGB(LED, InCh)       InCh_to_TmpVar(InCh, 4)      /* 29.10.19:  Old: 7 Input channels, 500 ms */  \
                                        XPatternT1(LED,12,SI_LocalVar,18,0,128,0,0,125 ms,0,240,0,15,0,0,0,0,0,240,0,0,0,0,0,0,0,0,240,0,0,0,0,0,0,240,15,0,240,0,0,16,17,17,1,0  ,63,191,191,191)

#define DepSignal4(LED, InCh)           InCh_to_TmpVar(InCh, 4)      /* 29.10.19:  Old: 7 Input channels, 500 ms */  \
                                        XPatternT1(LED,12,SI_LocalVar,5,0,128,0,0,125 ms,15,240,0,15,0,240,15,240,0,16  ,63,191,191,191)

// Entry signal with 3 aspects
#define EntrySignal3Bin_RGB(LED, InCh)  Bin_InCh_to_TmpVar(InCh, 2)  /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,160,SI_LocalVar,9,0,128,0,0,500 ms,64,4,104,0  ,63,191,191)

#define EntrySignal3Bin(LED, InCh)      Bin_InCh_to_TmpVar(InCh, 2)  /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,224,SI_LocalVar,3,0,128,0,0,125 ms,145,1  ,63,191,191)

// Departure signal with 4 aspects
#define DepSignal4Bin_RGB(LED, InCh)    Bin_InCh_to_TmpVar(InCh, 2)  /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,12,SI_LocalVar,18,0,128,0,0,125 ms,0,240,0,15,0,0,0,0,0,240,0,0,0,0,0,0,0,0,240,0,0,0,0,0,0,240,15,0,240,0,0,16,17,17,1,0  ,63,191,191,191)

#define DepSignal4Bin(LED, InCh)        Bin_InCh_to_TmpVar(InCh, 2)  /* 29.10.19:  Old: 500 ms */                    \
                                        XPatternT1(LED,12,SI_LocalVar,5,0,128,0,0,125 ms,15,240,0,15,0,240,15,240,0,16  ,63,191,191,191)




#define  ButtonFunc(                DstVar, InCh, Duration)                      Random(DstVar, InCh, RF_STAY_ON, 0, 0, (Duration), (Duration))             // DstVar is turned on if InCh is activated and stays on for duration (Static (Not Edge) retiggerable mono flop)

// Single DstVar (Q)
#define  MonoFlop(                  DstVar, InCh, Duration)                      Counter(_CM_RS_FlipFlop1,               InCh,   SI_0,   Duration, DstVar)  // Retrigger with rising endge (Like ButtonFunc, but edge triggered)
#define  MonoFlopLongReset(         DstVar, InCh, Duration)                      Counter(_CM_RS_FlipFlop1|CF_RESET_LONG, InCh,   SI_0,   Duration, DstVar)  // Retrigger with rising endge, Long press disables
#define  RS_FlipFlop(               DstVar, S_InCh, R_InCh)                      Counter(_CM_RS_FlipFlop1,               S_InCh, R_InCh, 0,        DstVar)  // Edge tiggered RS Flip Flop
#define  RS_FlipFlopTimeout(        DstVar, S_InCh, R_InCh, Timeout)             Counter(_CM_RS_FlipFlop1,               S_InCh, R_InCh, Timeout,  DstVar)  // Edge tiggered RS Flip Flop and timeout
#define  T_FlipFlopReset(           DstVar, T_InCh, R_InCh)                      Counter(_CM_T_FlipFlop1,                T_InCh, R_InCh, 0,        DstVar)  // Toggle Flip Flop with reset
#define  T_FlipFlopResetTimeout(    DstVar, T_InCh, R_InCh, Timeout)             Counter(_CM_T_FlipFlop1,                T_InCh, R_InCh, Timeout,  DstVar)  // Toggle Flip Flop with restart and timeout

// Single DstVar which is controlled inverse (Q')  => DstVar is active at the begining and when a timeout timeout occoures. It's deactivated with the trigger
#define  MonoFlopInv(               DstVar, InCh, Duration)                      Counter(_CM_RS_FlipFlop2,               InCh,   SI_0,   Duration, DstVar,  DstVar)  // Retrigger with rising endge (Like ButtonFunc, but edge triggered)
#define  MonoFlopInvLongReset(      DstVar, InCh, Duration)                      Counter(_CM_RS_FlipFlop2|CF_RESET_LONG, InCh,   SI_0,   Duration, DstVar,  DstVar)  // Retrigger with rising endge, Long press disables
#define  RS_FlipFlopInv(            DstVar, S_InCh, R_InCh)                      Counter(_CM_RS_FlipFlop2,               S_InCh, R_InCh, 0,        DstVar,  DstVar)  // Edge tiggered RS Flip Flop
#define  RS_FlipFlopInvTimeout(     DstVar, S_InCh, R_InCh, Timeout)             Counter(_CM_RS_FlipFlop2,               S_InCh, R_InCh, Timeout,  DstVar,  DstVar)  // Edge tiggered RS Flip Flop and timeout
#define  T_FlipFlopInvReset(        DstVar, T_InCh, R_InCh)                      Counter(_CM_T_FlipFlopReset2,           T_InCh, R_InCh, 0,        DstVar,  DstVar)  // Toggle Flip Flop with reset Q'
#define  T_FlipFlopInvResetTimeout( DstVar, T_InCh, R_InCh, Timeout)             Counter(_CM_T_FlipFlopReset2,           T_InCh, R_InCh, Timeout,  DstVar,  DstVar)  // Toggle Flip Flop with restart and timeout Q'

// Dual DstVar (Q and Q')
#define  MonoFlop2(                 DstVar0, DstVar1, InCh, Duration)            Counter(_CM_RS_FlipFlop2,               InCh,   SI_0,   Duration, DstVar0, DstVar1)  // Two Outputs, retrigger with rising endge (Like ButtonFunc, but edge triggered)
#define  MonoFlop2LongReset(        DstVar0, DstVar1, InCh, Duration)            Counter(_CM_RS_FlipFlop2|CF_RESET_LONG, InCh,   SI_0,   Duration, DstVar0, DstVar1)  // Two Outputs, retrigger with rising endge, Long press disables
#define  RS_FlipFlop2(              DstVar0, DstVar1, S_InCh, R_InCh)            Counter(_CM_RS_FlipFlop2,               S_InCh, R_InCh, 0,        DstVar0, DstVar1)  // Edge tiggered RS Flip Flop
#define  RS_FlipFlop2Timeout(       DstVar0, DstVar1, S_InCh, R_InCh, Timeout)   Counter(_CM_RS_FlipFlop2,               S_InCh, R_InCh, Timeout,  DstVar0, DstVar1)  // Edge tiggered RS Flip Flop and timeout
#define  T_FlipFlop2Reset(          DstVar0, DstVar1, T_InCh, R_InCh)            Counter(_CM_T_FlipFlopReset2,           T_InCh, R_InCh, 0,        DstVar0, DstVar1)  // Toggle Flip Flop with reset
#define  T_FlipFlop2ResetTimeout(   DstVar0, DstVar1, T_InCh, R_InCh, Timeout)   Counter(_CM_T_FlipFlopReset2,           T_InCh, R_InCh, Timeout,  DstVar0, DstVar1)  // Toggle Flip Flop with restart and timeout

#define  RGB_Heartbeat(LED)                                           \
             New_HSV_Group()                                          \
             APatternT2(LED,192,SI_1,1,0,255,0,PM_HSV,60 Sek,0 ms,1)  \
             APatternT1(LED, 194,SI_1,1,10,255,0,PM_HSV|PF_EASEINOUT,1 Sek,1)   // Nicht ganz aus gehen weil es sonst bei manchen Farben flackert

#define  RGB_Heartbeat2(LED, MinBrightness, MaxBrightness)            \
             New_HSV_Group()                                          \
             APatternT2(LED,192,SI_1,1,0,255,0,PM_HSV,60 Sek,0 ms,1)  \
             APatternT1(LED, 194,SI_1,1,MinBrightness,MaxBrightness,0,PM_HSV|PF_EASEINOUT,1 Sek,1)            // 18.01.19:


// Push Button functions which count the button press and activates temporary variables
// The button flashes n times if the button was pressed n times.
// Parameter:
//   B_LED:    LED Number of the LED in the PUSH button (Example NUM_LEDS_1)
//   B_LED_Cx: Channel of the LED (C1..C3)
//   InCh:     Number of the input channel which is used to read in the button
//   TmpNr:    First temporaty variable. There function PushButton_w_LED_0_2 uses 3 temporary variables. PushButton_w_LED_0_3 uses 4, ...
//   Rotate:   If the buton is pressed n+1 times the first temporary variable is activated again
//   Timeout:  Time when the push button action is disabled again
//

// 10.03.19:  Added B_LED_Cx to be able to use the functions with all button LEDs
#define Status_Button_0_2(LED,B_LED_Cx, Val1)  PatternT1(LED,232+_Cx2StCh(B_LED_Cx),SI_LocalVar,1,0,Val1,0,0,0.4 Sec,57,128,227,0,0  ,63,192,0,0,1,192,0,0,0,0,2)
#define Status_Button_0_3(LED,B_LED_Cx, Val1)  PatternT1(LED,232+_Cx2StCh(B_LED_Cx),SI_LocalVar,1,0,Val1,0,0,0.4 Sec,57,128,227,0,142,227,0,0  ,63,192,0,0,1,192,0,0,0,0,2,192,0,0,0,0,0,0,3)
#define Status_Button_0_4(LED,B_LED_Cx, Val1)  PatternT1(LED,40 +_Cx2StCh(B_LED_Cx),SI_LocalVar,1,0,Val1,0,0,0.4 Sec,57,128,227,0,142,227,0,142,227,56,0  ,63,192,0,0,1,192,0,0,0,0,2,192,0,0,0,0,0,0,3,192,0,0,0,0,0,0,0,0,4)
#define Status_Button_0_5(LED,B_LED_Cx, Val1)  PatternT1(LED,168+_Cx2StCh(B_LED_Cx),SI_LocalVar,1,0,Val1,0,0,0.4 Sec,57,128,227,0,142,227,0,142,227,56,128,227,56,142,3,0  ,63,192,0,0,1,192,0,0,0,0,2,192,0,0,0,0,0,0,3,192,0,0,0,0,0,0,0,0,4,192,0,0,0,0,0,0,0,0,0,0,5)

#define PushButton_w_LED_0_2(B_LED, B_LED_Cx, InCh, TmpNr, Rotate, Timeout)                                                                   \
            Counter((Rotate?CF_ROTATE:0)|CF_SKIP0|CF_RESET_LONG, InCh, SI_1, Timeout, TmpNr+0, TmpNr+1, TmpNr+2)                              \
            InCh_to_TmpVar(TmpNr, 2+1)                                                                                                        \
            Status_Button_0_2(B_LED, B_LED_Cx, 255)

#define PushButton_w_LED_0_3(B_LED, B_LED_Cx, InCh, TmpNr, Rotate, Timeout)                                                                   \
            Counter((Rotate?CF_ROTATE:0)|CF_SKIP0|CF_RESET_LONG, InCh, SI_1, Timeout, TmpNr+0, TmpNr+1, TmpNr+2, TmpNr+3)                     \
            InCh_to_TmpVar(TmpNr, 3+1)                                                                                                        \
            Status_Button_0_3(B_LED, B_LED_Cx,255)

#define PushButton_w_LED_0_4(B_LED, B_LED_Cx, InCh, TmpNr, Rotate, Timeout)                                                                   \
            Counter((Rotate?CF_ROTATE:0)|CF_SKIP0|CF_RESET_LONG, InCh, SI_1, Timeout, TmpNr+0, TmpNr+1, TmpNr+2, TmpNr+3, TmpNr+4)            \
            InCh_to_TmpVar(TmpNr, 4+1)                                                                                                        \
            Status_Button_0_4(B_LED, B_LED_Cx,255)

#define PushButton_w_LED_0_5(B_LED, B_LED_Cx, InCh, TmpNr, Rotate, Timeout)                                                                   \
            Counter((Rotate?CF_ROTATE:0)|CF_SKIP0|CF_RESET_LONG, InCh, SI_1, Timeout, TmpNr+0, TmpNr+1, TmpNr+2, TmpNr+3, TmpNr+4, TmpNr+5)   \
            InCh_to_TmpVar(TmpNr, 5+1)                                                                                                        \
            Status_Button_0_5(B_LED, B_LED_Cx,255)



//--------------------------------------- MP3-TF-16P Sound modul ---------------------------------------------

                                //  ADKey1  ADKey2
#define SOUND_ADKEY10	11		//	Play Mode	14
#define SOUND_ADKEY9	17		//	U/SD/SPI	13
#define SOUND_ADKEY8	22		//	Loop All	12
#define SOUND_ADKEY7	25		//	Pause/Play	11
#define SOUND_ADKEY6	29		//	Prev/Vol-	10
#define SOUND_ADKEY5	37		//	Next/Vol+	9
#define SOUND_ADKEY4    49      //  4           8
#define SOUND_ADKEY3    70      //  3           7
#define SOUND_ADKEY2    134     //  2           6
#define SOUND_ADKEY1    255     //  1           5


// Sound functions could be disable / enabled with the variable SI_Enable_Sound
#define Sound_ADKey(    LED, InCh, ADKey1, ADKey2)         PatternTE2(LED,28,InCh,SI_Enable_Sound,2,0,255,0,PM_SEQUENZ_NO_RESTART,200 ms,10 ms,ADKey1,ADKey2,0,0)

// Volume changing is independent from SI_Enable_Sound
#define Sound_ADKeyTime(LED, InCh, ADKey1, ADKey2, Time)   PatternT2( LED,28,InCh,                  2,0,255,0,PM_SEQUENZ_NO_RESTART,Time,10 ms,ADKey1,ADKey2,0,0)

#define Sound_Prev(      LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY5,  0)
#define Sound_Next(      LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY6,  0)
#define Sound_PausePlay( LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY7,  0)
#define Sound_Loop(      LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY8,  0)
#define Sound_USDSPI(    LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY9,  0)  // ? U / SD / SPI
#define Sound_PlayMode(  LED, InCh)         Sound_ADKey(LED, InCh, SOUND_ADKEY10, 0)  // If Loop is active the button toggles between  Sequence, Sequence, Repeat same, Random, Loop off

// Volume
// First Volume change after 1000 ms
// Then volume is changed every 150 ms
// According to the documentation there are 30 stepps
#define Sound_DecVol(    LED, InCh, Steps)  Sound_ADKeyTime(LED, InCh, SOUND_ADKEY5,  0, 850 ms - 75 ms + (Steps-1) * 150 ms)
#define Sound_IncVol(    LED, InCh, Steps)  Sound_ADKeyTime(LED, InCh, SOUND_ADKEY6,  0, 850 ms - 75 ms + (Steps-1) * 150 ms)

#define Sound_Seq1( LED, InCh)    Sound_ADKey(LED, InCh, SOUND_ADKEY1, 0)
#define Sound_Seq2( LED, InCh)    Sound_ADKey(LED, InCh, SOUND_ADKEY2, 0)
#define Sound_Seq3( LED, InCh)    Sound_ADKey(LED, InCh, SOUND_ADKEY3, 0)
#define Sound_Seq4( LED, InCh)    Sound_ADKey(LED, InCh, SOUND_ADKEY4, 0)
#define Sound_Seq5( LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY1)
#define Sound_Seq6( LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY2)
#define Sound_Seq7( LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY3)
#define Sound_Seq8( LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY4)
#define Sound_Seq9( LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY5)
#define Sound_Seq10(LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY6)
#define Sound_Seq11(LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY7)
#define Sound_Seq12(LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY8)
#define Sound_Seq13(LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY9)
#define Sound_Seq14(LED, InCh)    Sound_ADKey(LED, InCh, 0, SOUND_ADKEY10)

#define _LocalVar_Sound(LED)  PatternT1(LED,28,SI_LocalVar,2,0,255,0,0,200 ms,0,0,SOUND_ADKEY1,0,SOUND_ADKEY2,0,SOUND_ADKEY3,0,SOUND_ADKEY4,0,0,SOUND_ADKEY1,0,SOUND_ADKEY2,0,SOUND_ADKEY3,0,SOUND_ADKEY4,0,SOUND_ADKEY5,0,SOUND_ADKEY6,0,SOUND_ADKEY7,0,SOUND_ADKEY8,0,SOUND_ADKEY9,0,SOUND_ADKEY10,0,0  ,1,129,129,129,129,129,129,129,129,129,129,129,129,129,129,127)

#define Sound_PlayRandom(LED, InCh, MaxSoundNr)                                                                      \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_RANDOM | CF_SKIP0,   InCh, SI_1, 0 Sec, MaxSoundNr+1)                    \
              _LocalVar_Sound(LED)

#define Sound_Next_of_N_Reset(LED, InCh, InReset, MaxSoundNr)                                                        \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_ROTATE | CF_SKIP0 | CF_INV_ENABLE,   InCh, InReset, 0 Sec, MaxSoundNr+1) \
              _LocalVar_Sound(LED)

#define Sound_Next_of_N(LED, InCh, MaxSoundNr)                                                                       \
             Sound_Next_of_N_Reset(LED, InCh, SI_0, MaxSoundNr)

//---------------------------- JQ6500 Sound modul ---------------------------------                           // 13.02.19:

// Sound functions could be disable / enabled with the variable SI_Enable_Sound
#define Sound_JQ6500_ADKey(     LED, InCh, ADKey)         PatternTE2(LED,28,InCh,SI_Enable_Sound,1,0,255,0,PM_SEQUENZ_NO_RESTART,400 ms,10 ms,ADKey,0) // Normal WS2811 modul with G+RB outputs anf -O+ inputs

// Volume changing is independent from SI_Enable_Sound
#define Sound_JQ6500_ADKeyTime(     LED, InCh, ADKey, Time)   PatternT2( LED,28,InCh,                1,0,255,0,PM_SEQUENZ_NO_RESTART,Time,10 ms,ADKey,0)

                                        //  ADKey1     400ms 470 Ohm
#define SOUND_JQ6500_ADKEY1_N   29      //  1
#define SOUND_JQ6500_ADKEY2_N   40      //  2
#define SOUND_JQ6500_ADKEY3_N   50      //  3
#define SOUND_JQ6500_ADKEY4_N   60      //  4
#define SOUND_JQ6500_ADKEY5_N   80      //  5
#define SOUND_JQ6500_ADKEY6_N   162     //  Prev/Vol-
#define SOUND_JQ6500_ADKEY7_N   231     //  Next/Vol+

// WS2811 with swapped pins (R+GB/+O-)  //  ADKey1
#define SOUND_JQ6500_ADKEY1_BG	32		//	1
#define SOUND_JQ6500_ADKEY2_BG	44		//	2
#define SOUND_JQ6500_ADKEY3_BG	56		//	3
#define SOUND_JQ6500_ADKEY4_BG	68		//	4
#define SOUND_JQ6500_ADKEY5_BG	90		//	5
#define SOUND_JQ6500_ADKEY6_BG	183		//	Prev/Vol-
#define SOUND_JQ6500_ADKEY7_BG	238		//	Next/Vol+


#define Sound_JQ6500_Prev(  LED, InCh)         Sound_JQ6500_ADKey(     LED, InCh, SOUND_JQ6500_ADKEY6_N)
#define Sound_JQ6500_Next(  LED, InCh)         Sound_JQ6500_ADKey(     LED, InCh, SOUND_JQ6500_ADKEY7_N)

#define Sound_JQ6500_BG_Prev(  LED, InCh)      Sound_JQ6500_ADKey(     LED, InCh, SOUND_JQ6500_ADKEY6_BG)
#define Sound_JQ6500_BG_Next(  LED, InCh)      Sound_JQ6500_ADKey(     LED, InCh, SOUND_JQ6500_ADKEY7_BG)

// Volume
// First Volume change after ~1000 ms   ???
// Then volume is changed every 150 ms  ???
// 30 stepps ??

#define Sound_JQ6500_DecVol(LED, InCh, Steps)  Sound_JQ6500_ADKeyTime(LED, InCh, SOUND_JQ6500_ADKEY6_N,     1000 ms + (Steps-1) * 150 ms)
#define Sound_JQ6500_IncVol(LED, InCh, Steps)  Sound_JQ6500_ADKeyTime(LED, InCh, SOUND_JQ6500_ADKEY7_N,     1000 ms + (Steps-1) * 150 ms)

#define Sound_JQ6500_BG_DecVol(LED, InCh, Steps)  Sound_JQ6500_ADKeyTime(LED, InCh, SOUND_JQ6500_ADKEY6_BG, 1000 ms + (Steps-1) * 150 ms)
#define Sound_JQ6500_BG_IncVol(LED, InCh, Steps)  Sound_JQ6500_ADKeyTime(LED, InCh, SOUND_JQ6500_ADKEY7_BG, 1000 ms + (Steps-1) * 150 ms)


#define Sound_JQ6500_Seq1(  LED, InCh)         Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY1_N)
#define Sound_JQ6500_Seq2(  LED, InCh)         Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY2_N)
#define Sound_JQ6500_Seq3(  LED, InCh)         Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY3_N)
#define Sound_JQ6500_Seq4(  LED, InCh)         Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY4_N)
#define Sound_JQ6500_Seq5(  LED, InCh)         Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY5_N)

#define Sound_JQ6500_BG_Seq1(  LED, InCh)      Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY1_BG)
#define Sound_JQ6500_BG_Seq2(  LED, InCh)      Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY2_BG)
#define Sound_JQ6500_BG_Seq3(  LED, InCh)      Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY3_BG)
#define Sound_JQ6500_BG_Seq4(  LED, InCh)      Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY4_BG)
#define Sound_JQ6500_BG_Seq5(  LED, InCh)      Sound_JQ6500_ADKey(LED, InCh, SOUND_JQ6500_ADKEY5_BG)

#define _LocalVar_Sound_JQ6500(   LED)    PatternT1(LED,28,SI_LocalVar,1,0,255,0,0,400 ms,0,SOUND_JQ6500_ADKEY1_N, SOUND_JQ6500_ADKEY2_N, SOUND_JQ6500_ADKEY3_N, SOUND_JQ6500_ADKEY4_N, SOUND_JQ6500_ADKEY5_N, 0,1,129,129,129,129,129,127)
#define _LocalVar_Sound_JQ6500_BG(LED)    PatternT1(LED,28,SI_LocalVar,1,0,255,0,0,400 ms,0,SOUND_JQ6500_ADKEY1_BG,SOUND_JQ6500_ADKEY2_BG,SOUND_JQ6500_ADKEY3_BG,SOUND_JQ6500_ADKEY4_BG,SOUND_JQ6500_ADKEY5_BG,0,1,129,129,129,129,129,127)


#define Sound_JQ6500_PlayRandom(LED, InCh, MaxSoundNr)                                                               \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_RANDOM | CF_SKIP0,   InCh, SI_1, 0 Sec, MaxSoundNr+1)                    \
              _LocalVar_Sound_JQ6500(LED)

#define Sound_JQ6500_Next_of_N_Reset(LED, InCh, InReset, MaxSoundNr)                                                 \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_ROTATE | CF_SKIP0 | CF_INV_ENABLE,   InCh, InReset, 0 Sec, MaxSoundNr+1) \
              _LocalVar_Sound_JQ6500(LED)

#define Sound_JQ6500_Next_of_N(LED, InCh, MaxSoundNr)                                                                \
             Sound_JQ6500_Next_of_N_Reset(LED, InCh, SI_0, MaxSoundNr)


#define Sound_JQ6500_BG_PlayRandom(LED, InCh, MaxSoundNr)                                                            \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_RANDOM | CF_SKIP0,   InCh, SI_1, 0 Sec, MaxSoundNr+1)                    \
              _LocalVar_Sound_JQ6500_BG(LED)

#define Sound_JQ6500_BG_Next_of_N_Reset(LED, InCh, InReset, MaxSoundNr)                                              \
              New_Local_Var()                                                                                        \
              Counter(CF_ONLY_LOCALVAR | CF_ROTATE | CF_SKIP0 | CF_INV_ENABLE,   InCh, InReset, 0 Sec, MaxSoundNr+1) \
              _LocalVar_Sound_JQ6500_BG(LED)

#define Sound_JQ6500_BG_Next_of_N(LED, InCh, MaxSoundNr)                                                             \
             Sound_JQ6500_BG_Next_of_N_Reset(LED, InCh, SI_0, MaxSoundNr)

/*
#define Sound_Seq( LED, InCh, SeqNr, SoundModul, WS2811_Typ) _Sound##SoundModul##_ADKey(    LED, InCh, SOUND##SoundModul##_ADKEY##SeqNr##WS2811_Typ)
#define Sound_Pre( LED, InCh,        SoundModul, WS2811_Typ) _Sound##SoundModul##_ADKey(    LED, InCh, SOUND##SoundModul##_ADKEY6##WS2811_Typ)
#define Sound_Nxt( LED, InCh,        SoundModul, WS2811_Typ) _Sound##SoundModul##_ADKey(    LED, InCh, SOUND##SoundModul##_ADKEY7##WS2811_Typ)
#define Sound_DecV(LED, InCh, Steps, SoundModul, WS2811_Typ) _Sound##SoundModul##_ADKeyTime(LED, InCh, SOUND##SoundModul##_ADKEY6##WS2811_Typ, 1000 ms + (Steps-1) * 150 ms)
#define Sound_IncV(LED, InCh, Steps, SoundModul, WS2811_Typ) _Sound##SoundModul##_ADKeyTime(LED, InCh, SOUND##SoundModul##_ADKEY7##WS2811_Typ, 1000 ms + (Steps-1) * 150 ms)
*/

#define EndCfg END_T



#define MobaLedLib_Configuration()          const PROGMEM unsigned char Config[] =

#define MobaLedLib_Create(leds)             uint8_t Config_RAM[__COUNTER__/2]; /* RAM used for the configuration functions. The size is calculated in the macros which are used in the Config[] table.*/ \
                                            MobaLedLib_C MobaLedLib(leds, sizeof(leds)/sizeof(CRGB), Config, Config_RAM, sizeof(Config_RAM)); // MobaLedLib_C class definition
#if _USE_USE_GLOBALVAR
  #define MobaLedLib_Assigne_GlobalVar(GlobalVar)  MobaLedLib.Assigne_GlobalVar(GlobalVar, (uint8_t)(sizeof(GlobalVar)/sizeof(ControlVar_t)))
#endif

#define _ConvChannelNr2Byte(ChannelNr) (ChannelNr%4 == 0 ? (ChannelNr)/4 : 512 + (ChannelNr)/4) // Generates a warning if the channel number can not be divided by 4

#define MobaLedLib_Copy_to_InpStruct(Src, ByteCnt, ChannelNr)  \
             MobaLedLib.Copy_Bits_to_InpStructArray(Src, ByteCnt, _ConvChannelNr2Byte(ChannelNr)) // Attention: Channel number must be divisible by 4

// Die Cx Konstanten bestehen aus:
// - LedCnt (1..3), StCh (0..2)
#define             _Cx(LedCnt, StCh)   (LedCnt + (StCh<<2))
#define C1          _Cx(1,      0)
#define C2          _Cx(1,      1)
#define C3          _Cx(1,      2)
#define C12         _Cx(2,      0)
#define C23         _Cx(2,      1)
#define C_ALL       _Cx(3,      0)
#define C_RED       C1
#define C_GREEN     C2
#define C_BLUE      C3
#define C_WHITE     C_ALL
#define C_YELLOW    C12
#define C_CYAN      C23

#define _Cx2LedCnt(Cx) ((Cx)&0x03)
#define _Cx2StCh(Cx)   (((Cx)>>2)&0x03)

// FreeBits ist abhaengig vom Muster:
// - Andreaskr.  2 LEDs 0   01101001   Typ:0
// - Const,      1 LED  7   .......1   Typ:1
// - Const,      2 LEDs 6   ......11
// - Const,      3 LEDs 5   .....111
// - Blinker,    1 LED  6   ......01   Typ:2
// - Blinker,    2 LEDs 4   ....0011
// - Blinker,    3 LEDs 2   ..000111
// - DoublFlash, 1 LED  4   ....0101   Typ:4
// - DoublFlash, 2 LEDs 0   00110011
// - DoublFlash, 3 LEDs 4   ....0001 11000111  <= Geht nicht mit dem gleichen Makro -> LeuchtfeuerALL
#define _FreeBits(LedCnt, Typ) ((Typ<=2?(8-(LedCnt)*Typ):((LedCnt)==2?0:4))&0x07)

// NStru contains the Start Channel:0-2 (Bits 0,1), the Bits per Channel-1: 1-8 (Bits 2..4) and the Number of free Bits: 0-7 (Bits 5..7)
// Picture NNNBBBSS
#define _NStru(cx, Typ, BpC) (_Cx2StCh(cx) + ((BpC-1)<<2) + (_FreeBits(_Cx2LedCnt(cx), Typ)<<5))

// The BlueLight and the Leuchtfeuer macros can't be used with C_ALL because in this case
// the two byte pattern B11000111, B00000001 is needed. I don't know how to generate this
// with the macro PAT_DBLFL. Therefore the following strange number is used which is printed
// as a warning. Maybe this helps to find this comment ...
#define _C_ALL_not_supported  33877u // Generates the warning "narrowing conversion of '33877u' from 'unsigned int' to ..."


#define _PAT_BLINK(LedCnt) ((LedCnt)==1?B00000001: ((LedCnt)==2?B00000011:B00000111))
#define _PAT_DBLFL(LedCnt) ((LedCnt)==1?B00000101: ((LedCnt)==2?B00110011:_C_ALL_not_supported))  // Doesn't work with C_ALL ;-( because in this case we need 2 parameters: B11000111, B00000001

#define _Cx2P_BLINK(Cx)   (_PAT_BLINK(_Cx2LedCnt(Cx)))
#define _Cx2P_DBLFL(Cx)   (_PAT_DBLFL(_Cx2LedCnt(Cx)))


// Constants for the Button period and the times in the pattern function
#define Min         *60000L    // min in lowercase is not possible because of the standard min macro
#define Sec         *1000L
#define Sek         Sec
#define sek         Sec
#define sec         Sec
#define Ms
#define ms

#define PM_NORMAL               0              // Normal mode (Als Platzhalter in Excel)
#define PM_SEQUENZ_W_RESTART    1              // Rising edge-triggered unique sequence. A new edge starts with state 0.
#define PM_SEQUENZ_W_ABORT      2              // Rising edge-triggered unique sequence. Abort the sequence if new edge is detected during run time.
#define PM_SEQUENZ_NO_RESTART   3              // Rising edge-triggered unique sequence. No restart if new edge is detected
#define PM_SEQUENZ_STOP         4              // Rising edge-triggered unique sequence. A new edge starts with state 0. If input is turned off the sequence is stopped immediately.
#define PM_PINGPONG             5              // Change the direction at the end: 118 bytes
#define PM_HSV                  6              // Use HSV values instead of RGB for the channels
#define PM_RES                  7              // Reserved mode
#define PM_MODE_MASK            0x07           // Defines the number of bits used for the modes (currently 3 => Modes 0..7)

// Flags for the Pattern function
#define _PF_XFADE               0x08           // Special fade mode which starts from the actual brightness value instead of the value of the previous state
#define PF_NO_SWITCH_OFF        0x10           // Don't switch of the LEDs if the input is turned off. Useful if several effects use the same LEDs alternated by the input switch.
#define PF_EASEINOUT            0x20           // Easing function (Uebergangsfunktion) is used because changes near 0 and 255 are noticed different than in the middle
#define PF_SLOW                 0x40           // Slow timer (divided by 16) to be able to use longer durations
#define PF_INVERT_INP           0x80           // Invert the input switch => Effect is active if the input is 0

// Flags and modes for the Random() and RandMux() function
#define  RM_NORMAL               B00000000
#define  RF_SLOW                 B00000001      // Time base is divided by 16 This Flag is set automatically if the time is > 65535 ms
#define  RF_SEQ                  B00000010      // Switch the outputs of the RandMux() function sequential and not random
#define  RF_STAY_ON              B00000100      // Flag for the Ranom() function. The Output stays on until the input is turned off. MinOn, MaxOn define how long it stays on.
                                                // 12.01.20:  For some reasons RF_STAY_ON was equal RF_SEQ
#define  RF_NOT_SAME             B00001000      // Prevent, that the same output is used two times in sequence                                           // 13.01.20:

// Flags for the Counter() function
#define  CM_NORMAL               0              // Normal Counter mode
#define  CF_INV_INPUT            B00000001      // Invert Input
#define  CF_INV_ENABLE           B00000010      // Input Enable
#define  CF_BINARY               B00000100      // Maximal 8 outputs
#define  CF_RESET_LONG           B00001000      // Taste Lang = Reset
#define  CF_UP_DOWN              B00010000      // Ein RS-FlipFlop kann mit CM_UP_DOWN ohne CF_ROTATE gemacht werden
#define  CF_ROTATE               B00100000      // Faengt am Ende wieder von vorne an
#define  CF_PINGPONG             B01000000      // Wechselt am Ende die Richtung
#define  CF_SKIP0                B10000000      // Ueberspringt die 0. Die 0 kommt nur bei einem Timeout oder wenn die Taste lange gerueckt wird
#define  CF_RANDOM              (B00000001<<8)  // Generate random numbers                                                                               // 07.11.18:
#define  CF_LOCAL_VAR           (B00000010<<8)  // Write the result to a local variable which must be created with New_Local_Var() prior
#define _CF_NO_LEDOUTP          (B00000100<<8)  // Disable the LED output (the first DestVar contains the maximal counts-1 (counter => 0 .. n-1) )
#define  CF_ONLY_LOCALVAR       (CF_LOCAL_VAR|_CF_NO_LEDOUTP)  // Don't write to the LEDs defined with DestVar. The first DestVar contains the number maximal number of the counter-1 (counter => 0 .. n-1).


#define _CM_RS_FlipFlop1        (CF_BINARY|CF_UP_DOWN)               // RS Flip Flop with one output (Edge triggered)
#define _CM_T_FlipFlop1         (CF_BINARY|CF_INV_ENABLE|CF_ROTATE)  // T Flip Flop  with one output
#define _CM_RS_FlipFlop2        (CF_UP_DOWN)                         // RS Flip Flop with two outputs (Edge triggered)
#define _CM_T_FlipFlopEnable2   (CF_ROTATE)                          // T Flip Flop  with two outputs and enable
#define _CM_T_FlipFlopReset2    (CF_ROTATE | CF_INV_ENABLE)          // T Flip Flop  with two outputs and reset

#define _CF_ROT_SKIP0           (CF_ROTATE   | CF_SKIP0)             // Rotate and Skip 0
#define _CF_P_P_SKIP0           (CF_PINGPONG | CF_SKIP0)



// Constants for the rooms in the HOUSE_T
#define ROOM_DARK          0         // Dark light
#define ROOM_BRIGHT        1         // Bright light
#define ROOM_WARM_W        2         // Warm white light
#define ROOM_RED           3         // Red light
#define ROOM_D_RED         4         // Dark Red light
#define ROOM_COL0          5         // Room with user defined color 0
#define ROOM_COL1          6         // Room with user defined color 1
#define ROOM_COL2          7         // Room with user defined color 2
#define ROOM_COL3          8         // Room with user defined color 3
#define ROOM_COL4          9         // Room with user defined color 4
#define ROOM_COL5          10        // Room with user defined color 5
#define ROOM_COL345        11        // Room with user defined color 3, 4 or 5 which is randomly activated
#define FIRE              (12 +RAM1) // Chimney fire   (RAM is used to store the Heat_p)
#define FIRED             (13 +RAM1) // Dark chimney     "
#define FIREB             (14 +RAM1) // Bright chimney   "
// Reserved don't remove   15
#define ROOM_CHIMNEY      (16 +RAM1) // With chimney fire or Light        (RAM is used to store the Heat_p for the chimney)
#define ROOM_CHIMNEYD     (17 +RAM1) // With dark chimney fire or Light     "
#define ROOM_CHIMNEYB     (18 +RAM1) // With bright chimney fire or Light   "
#define ROOM_TV0           19        // With TV channel 0 or Light
#define ROOM_TV0_CHIMNEY  (20 +RAM1) // With TV channel 0 and fire or Light
#define ROOM_TV0_CHIMNEYD (21 +RAM1) // With TV channel 0 and fire or Light
#define ROOM_TV0_CHIMNEYB (22 +RAM1) // With TV channel 0 and fire or Light
#define ROOM_TV1           23        // With TV channel 1 or Light
#define ROOM_TV1_CHIMNEY  (24 +RAM1) // With TV channel 1 and fire or Light
#define ROOM_TV1_CHIMNEYD (25 +RAM1) // With TV channel 1 and fire or Light
#define ROOM_TV1_CHIMNEYB (26 +RAM1) // With TV channel 1 and fire or Light
#define GAS_LIGHT          27        // Gas light using all channels (RGB)           ATTENTION: First GAS lights are bright  (Don't mix them up)
#define GAS_LIGHT1         28        // Gas light using one channel (R)
#define GAS_LIGHT2         29        // Gas light using one channel (G)
#define GAS_LIGHT3         30        // Gas light using one channel (B)
#define GAS_LIGHTD         31        // Dark gas light using all channels (RGB)      ATTENTION: Following GAS lights are dark (Don't mix them up)
#define GAS_LIGHT1D        32        // Dark gas light using one channel (R)
#define GAS_LIGHT2D        33        // Dark gas light using one channel (G)
#define GAS_LIGHT3D        34        // Dark gas light using one channel (B)
#define NEON_LIGHT         35        // Neon light using all channels
#define NEON_LIGHT1        36        // Neon light using one channel (R)
#define NEON_LIGHT2        37        // Neon light using one channel (G)
#define NEON_LIGHT3        38        // Neon light using one channel (B)
#define NEON_LIGHTD        39        // Dark Neon light using all channels
#define NEON_LIGHT1D       40        // Dark Neon light using one channel (R)
#define NEON_LIGHT2D       41        // Dark Neon light using one channel (G)
#define NEON_LIGHT3D       42        // Dark Neon light using one channel (B)
#define NEON_LIGHTM        43        // Medium Neon light using all channels
#define NEON_LIGHT1M       44        // Medium Neon light using one channel (R)
#define NEON_LIGHT2M       45        // Medium Neon light using one channel (G)
#define NEON_LIGHT3M       46        // Medium Neon light using one channel (B)
#define NEON_LIGHTL        47        // Large room Neon light using all channels. A large room is equipped with several neon lights which start delayed
#define NEON_LIGHT1L       48        // Large room Neon light using one channel (R)
#define NEON_LIGHT2L       49        // Large room Neon light using one channel (G)
#define NEON_LIGHT3L       50        // Large room Neon light using one channel (B)
#define NEON_DEF_D         51        // Decective Neon light using all channels.
#define NEON_DEF1D         52        // Decective Neon light using one channel (R)
#define NEON_DEF2D         53        // Decective Neon light using one channel (G)
#define NEON_DEF3D         54        // Decective Neon light using one channel (B)
#define SINGLE_LED1        55        // Single LED connected with a WS2811 module to channel (R)              // 06.09.19:
#define SINGLE_LED2        56        // Single LED connected with a WS2811 module to channel (G)
#define SINGLE_LED3        57        // Single LED connected with a WS2811 module to channel (B)
#define SINGLE_LED1D       58        // Single dark LED connected with a WS2811 module to channel (R)
#define SINGLE_LED2D       59        // Single dark LED connected with a WS2811 module to channel (G)
#define SINGLE_LED3D       60        // Single dark LED connected with a WS2811 module to channel (B)         // 06.09.19:  End of new block
#define SKIP_ROOM          61        // Room which is not controlled with by the house() function (Usefull for Shops in a house becaue this lights are always on at night)

#if ((FIRE              & 0x03) != 0) || ((FIRED             & 0x03) != 1) || ((FIREB             & 0x03) != 2) ||   \
    ((ROOM_CHIMNEY      & 0x03) != 0) || ((ROOM_CHIMNEYD     & 0x03) != 1) || ((ROOM_CHIMNEYB     & 0x03) != 2) ||   \
    ((ROOM_TV0_CHIMNEY  & 0x03) != 0) || ((ROOM_TV0_CHIMNEYD & 0x03) != 1) || ((ROOM_TV0_CHIMNEYB & 0x03) != 2) ||   \
    ((ROOM_TV1_CHIMNEY  & 0x03) != 0) || ((ROOM_TV1_CHIMNEYD & 0x03) != 1) || ((ROOM_TV1_CHIMNEYB & 0x03) != 2)
    #warning Wrong number for ROOM_CHIMNEY, ... Last two bits are used in Update_TV_or_Chimney()
#endif



// Spezial Inputs
#define SI_Enable_Sound    253  // Wird auf 1 initialisiert, kann aber vom der Konfigurarion veraendert werden
#define SI_LocalVar        SI_0 // Input fuer Pattern Funktion zum einlesen der Localen Variable. Diese Nummer wird verwendet weil eine Konstante 0 als Eingang keinen Sinn macht
#define SI_0               254  // Immer 0
#define SI_1               255  // Immer 1

/*
 Weitere Special Inputs:   ??
 SI_Status_OK                  // Alles O.K.
 SI_Status_LED_Mis             // Weniger LEDs erkannt als vorgegeben
 SI_Status_Add_LED             // Mehr LEDs erkannt als vorgegeben
 SI_Status_CAN_OK              // CAN DAten vorhanden
 SI_Sunrise                    //
 SI_Sunset
 SI_Day
 SI_Night
*/

// Constants for the Logic() function
#define _OR_NR             SI_0      // Number which can't be used as input channel for the logic function ( A & false is always false)
#define _NOT_NR            SI_1      //  "                "                 "
#define OR                 ,_OR_NR,
#define NOT                _NOT_NR,
#define AND                ,
#define ENABLE             _OR_NR,   // Used to define an enable input. This keyword must be used at the first position in the variable argument list. An OR at this position dosn't make sence => we could use the same numner
#define DISABLE            _OR_NR, NOT

// 'or', 'and', ... is not possible because they are used in c++
// A macro EQ:',' is not possible because then it can't be checked in the macro

// Bit masks for Special SpInp[]
#define _BIT_OLD           B00000010
#define _BIT_NEW           B00000001
#define _NEW_AND_OLD       (_BIT_NEW + _BIT_OLD)

#define _ALL_OLD_BITS      B10101010
#define _ALL_NEW_BITS      B01010101

// Results generated by Get_Input()
#define INP_OFF            B00000000
#define INP_ON             B00000011
#define INP_TURNED_ON      B00000001
#define INP_TURNED_OFF     B00000010
#define INP_INIT_OFF       B00000110



//---------------------------------------------
inline bool Inp_Changed_or_InitOff(uint8_t Inp)
//---------------------------------------------
{
  return Inp == INP_TURNED_ON || Inp == INP_TURNED_OFF || Inp == INP_INIT_OFF;
}
// Optimierungsuntersuchung:
// Inp_Changed_or_InitOff():
//   - Wenn man nur den ersten Vergleich (Inp == INP_TURNED_ON) aktiviert, dann dauert
//     der Update 189us anstelle von 193us bei einer Konfiguration mit 13 Pattern aufrufen
//     und einer Logic() Funktion. => 2% schneller.
//   - Ohne die (Inp == INP_INIT_OFF) Abfrage dauert es 190us => 1.5% schneller.

//------------------------------------
inline uint8_t Invert_Inp(uint8_t Inp)
//------------------------------------
{
  return Inp ^ B00000011;  // XOR: 00 => 11, 01 => 10, 10 => 01, 11 => 00
}

//--------------------------------
inline bool Inp_Is_On(uint8_t Inp)
//--------------------------------
{
  return Inp == INP_ON || Inp == INP_TURNED_ON; //Inp & _BIT_NEW;
}

//--------------------------------------------------
inline bool Inp_Is_TurnedOff_or_InitOff(uint8_t Inp)
//--------------------------------------------------
{
  return Inp == INP_TURNED_OFF || Inp == INP_INIT_OFF ;
}

//----------------------------------
inline bool Inp_Changed(uint8_t Inp)
//----------------------------------
{
  return Inp == INP_TURNED_ON || Inp == INP_TURNED_OFF;
}

//********************************* Internal used data *****************************


// Die Verschiedenen Ausgangstypen:
#define END_T                  0
#define CONST_T                1 // Die ersten Ausgangstypen haben als Parameter eine channel mask mit
#define RANDMUX_T              2 // der ausgewaehlt wird welche LEDs angestuert werden
#define SCHEDULE_T             3 // Bei diesen Typen steht InCh an Position 2
#define PATTERNT1_T           10 // Muster mit einer Zeit fuer alle Zustaende
#define PATTERNT2_T           11 // Muster mit zwei abwechselnd genutzten Zeiten
#define PATTERNT3_T           12 //   "        drei     "
#define PATTERNT4_T           13
#define PATTERNT5_T           14
#define PATTERNT6_T           15
#define PATTERNT7_T           16
#define PATTERNT8_T           17
#define PATTERNT9_T           18
#define PATTERNT10_T          19
#define PATTERNT11_T          20
#define PATTERNT12_T          21
#define PATTERNT13_T          22
#define PATTERNT14_T          23
#define PATTERNT15_T          24
#define PATTERNT16_T          25
#define PATTERNT17_T          26
#define PATTERNT18_T          27
#define PATTERNT19_T          28
#define PATTERNT20_T          29
#define PATTERNT21_T          30
#define PATTERNT22_T          31
#define PATTERNT23_T          32
#define PATTERNT24_T          33
#define PATTERNT25_T          34
#define PATTERNT26_T          35
#define PATTERNT27_T          36
#define PATTERNT28_T          37
#define PATTERNT29_T          38
#define PATTERNT30_T          39
#define LAST_PATTERN_T        PATTERNT30_T

#define APATTERNT1_T          40 // Muster mit einer Zeit fuer alle Zustaende mit analogen Uebergaengen
#define APATTERNT2_T          41 // Muster mit zwei abwechselnd genutzten Zeiten
#define APATTERNT3_T          42 //   "        drei    "
#define APATTERNT4_T          43
#define APATTERNT5_T          44
#define APATTERNT6_T          45
#define APATTERNT7_T          46
#define APATTERNT8_T          47
#define APATTERNT9_T          48
#define APATTERNT10_T         49
#define APATTERNT11_T         50
#define APATTERNT12_T         51
#define APATTERNT13_T         52
#define APATTERNT14_T         53
#define APATTERNT15_T         54
#define APATTERNT16_T         55
#define APATTERNT17_T         56
#define APATTERNT18_T         57
#define APATTERNT19_T         58
#define APATTERNT20_T         59
#define APATTERNT21_T         60
#define APATTERNT22_T         61
#define APATTERNT23_T         62
#define APATTERNT24_T         63
#define APATTERNT25_T         64
#define APATTERNT26_T         65
#define APATTERNT27_T         66
#define APATTERNT28_T         67
#define APATTERNT29_T         68
#define APATTERNT30_T         69
#define LAST_APATTERN_T APATTERNT30_T

#define HOUSE_T               70 // Erster Ausgangstyp ohne ChMsk (WITHOUT_CHANNEL_MASK). Hier steht InCh an Position 1. Bei den Typen davor steht InCh an Position 2.
#define FIRE_T                71
#define RANDOM_T              72
#define WELDING_T             73
#define COPYLED_T             74
#define COUNTER_T             75
#define NEW_HSV_GROUP_T       100  // Erster Ausgangstyp ohne InpCh (WITHOUT_INP_CH)
#define NEW_LOCAL_VAR_T       101                                                                             // 07.11.18:
#define USE_GLOBALVAR_T       102
#define INCH_TO_TMPVAR_T      103                                                                             // 25.11.18:
#define BIN_INCH_TO_TMPVAR_T  104                                                                             // 18.01.19:
#ifdef _NEW_ROOM_COL
 #define SET_COLTAB_T         105
#endif
#if _USE_SET_TVTAB                                                                                            // 10.01.20:
 #define SET_TV_TAB_T         106
#endif

#if _USE_DEF_NEON                                                                                             // 12.01.20:
 #define SET_DEF_NEON_T       107
#endif

#define LOGIC_T               120

// Only used in the Charliplexing modul
#define CPX_LED_ASSIGNEMENT   140  // Define the LED assignement
#define CPX_ANALOG_INPUT      141  // Enable the analog input
#define CPX_ANALOG_LIMMITS    142  // Table with analog limits which defines the Goto Nr levels

#define WITHOUT_CHANNEL_MASK  HOUSE_T
#define WITHOUT_INP_CH        NEW_HSV_GROUP_T




#define _TV_CHANNELS          2  // If changed adapt also the constants ROOM_TV0, COLOR_TV0_A... and the corrosponding switch case statements

#include "Macros.h"


typedef struct
    {
    #if _USE_SET_TVTAB                                                                                        // 10.01.20:
      union {
            struct {
                   uint8_t r, g, b;
                   };
            uint8_t raw [3];
            };
    #else
      uint8_t         r, g, b;
    #endif
      uint8_t         Last_t;
      uint8_t         dt;
    } TV_Dat_T;     // 5 Byte

#ifdef _USE_LOCAL_TX
  #define CALCULATE_t4w  uint16_t t4w = (t>>4)  & 0xFFFF; // Time divided by 16
  #define CALCULATE_t4   uint8_t  t4  = (t>>4)  & 0xFF;   // Time divided by 16
  #define CALCULATE_t10  uint8_t  t10 = (t>>10) & 0xFF;   // Time divided by 1024
#else
  #define CALCULATE_t4w
  #define CALCULATE_t4
  #define CALCULATE_t10
#endif

#define ALL_CHANNELS    4   // For RawNr. Is used with "& 0x03" to select channel 0


#define TM_DISABLED     0
#define TM_LED_TEST     1   // Don't change the sequence because its needed for TestName[3][4]
#define TM_INP_TEST     2
#define TM_BUT_TEST     3   // Reserved mode, currently no function (If the line is enabled the mode could be selected with the middle key)


extern uint8_t    Darkness;
extern DayState_E DayState;

extern uint8_t    TestMode;

#define ROOM_COL_CNT    17   // Number of user defined colors for the room lights (Attention: If changed the program has to be adapted at several positions)  06.09.19:  Old 15

#define COLOR0         0
#define COLOR1         1
#define COLOR2         2
#define COLOR3         3
#define COLOR4         4
#define COLOR5         5
#define COLOR_GASL_D   6
#define COLOR_GASL_B   7
#define COLOR_NEON_D   8
#define COLOR_NEON_M   9
#define COLOR_NEON_B   10
#define COLOR_TV0_A    11
#define COLOR_TV0_B    12
#define COLOR_TV1_A    13
#define COLOR_TV1_B    14
#define COLOR_SINGLE   15                                                                                     // 06.09.19:
#define COLOR_SINGLE_D 16                                                                                     //   "



//:::::::::::::::::::
class MobaLedLib_C
//:::::::::::::::::::
{
public:
                    MobaLedLib_C(struct CRGB* _leds, uint16_t Num_Leds, const uint8_t Config[], uint8_t RAM[], uint16_t RamSize); // Construktor
#if _USE_USE_GLOBALVAR
 void               Assigne_GlobalVar(ControlVar_t *GlobalVar, uint8_t GlobalVar_Cnt);
#endif
 void               Update();
 void               Set_Input(uint8_t channel, uint8_t On);
 int8_t             Get_Input(uint8_t channel);
 void               Copy_Bits_to_InpStructArray(uint8_t *Src, uint8_t ByteCnt, uint8_t InpStructArrayNr);
 void               Print_Config(); // FLASH usage ~4258 Byte RAM 175 (16.12.18), #define _PRINT_DEBUG_MESSAGES must be enabled in "Lib_Config.h"

public:  // Variables

private: // Variables
 const uint8_t     *cp;          // Pointer to the Config[]            //  2 Byte
 CRGB*              leds;                                              //  2 Byte
 uint16_t           Num_Leds;                                          //  2 Byte
 const uint8_t     *Config;                                            //  2 Byte
 bool               Initialize;                                        //  1 Byte
 uint8_t           *RAM;                                               //  2 Byte
 uint8_t            Last_20;     // Last time for the 20fps flag       //  1 Byte
 uint8_t            TriggerCnt;                                        //  1 Byte
 uint8_t           *rp;          // Pointer to the RAM                 //  2 Byte
 uint32_t           t;           // actual time [ms]                   //  4 Byte
#ifndef _USE_LOCAL_TX
 uint16_t           t4w;         // time>>4   = / 16                   //  2 Byte
 uint8_t            t10;         // time>>10  = / 1024                 //  1 Byte
#endif
 uint8_t            Trigger20fps;                                      //  1 Byte
 TV_Dat_T           TV_Dat[_TV_CHANNELS];                              // 10 Byte
 uint8_t            InpStructArray[_INP_STRUCT_ARRAY_SIZE];            // 64 Byte   Array which contains two bits for each input. One bit for the actual state and a second for the new state. (See INP_TURNED_ON)
 HSV_T             *HSV_p;                                             //  2 Byte
 ControlVar_t      *ActualVar_p;                                       //  2 Byte                                                                  // 07.11.18:
#if _USE_USE_GLOBALVAR
 ControlVar_t      *GlobalVar;                                         //  2 Byte
#endif
 ControlVar_t       TempVar;                                           //  2 Byte
 uint8_t            GlobalVar_Cnt;                                     //  1 Byte
 Sound_Dat_t       *Sound_Dat;                                         //  2 Byte
                                                                       // 47 Byte RoomCol[]
                                                                       // -------
                                                                       //154 Byte

#if _USE_DEF_NEON                                                                                             // 12.01.20:
 uint8_t            Rand_On_DefNeon; // probability that the neon light starts.             0 = don't start, 1 start seldom,  255 = Start immediately
 uint8_t            RandOff_DefNeon; // probability that the light goes off after a while.  0 = Always on,   1 = shot flash,  255 = very long time active
 uint8_t            Min_DefNeon;     // red glow of the starter.                            1 = minimal value,  5 = maximal value
#endif


 // Private functions
 uint8_t           *Get_LEDPtr(uint8_t &cnt);
 uint8_t            Is_Room_On(CRGB *lp, uint8_t RawNr);
 void               Update_TV_or_Chimney(uint8_t TVNr, CRGB *lp, uint8_t Room_Typ);
 uint8_t            Get_RawNr(uint8_t Room_Typ);
 void               Update_Gas_Light( uint8_t Room_Typ, CRGB *lp);
 void               Update_Neon_Light(uint8_t Room_Typ, CRGB *lp);
 void               TurnOnRoom(CRGB* lp, uint8_t Room_Typ);
 void               TurnOffRoom(CRGB* lp, uint8_t RawNr, uint8_t Room_Typ);
 void               Set_Default_TV_Dat_p();                                                                   // 09.01.20:
 void               Update_TV_Data();
 void               Random_Const_Light(CRGB *lp, uint8_t TVNr);
 void               Enable_Light_or_TV(CRGB *lp, uint8_t TVNr);
 void               Enable_Chimney(CRGB *lp);
 void               Enable_Light_or_Chimney(CRGB *lp);
 void               Enable_Light_TV_or_Chimney(CRGB *lp, uint8_t TVNr);
 void               Set_Timer4Random(uint16_t Timer, TimerData16_T *dp, uint8_t P_MinTime);
 void               Set_Dest_RandMux(uint8_t Act, uint8_t Mode);
 void               Inp_Processed();
 void               CalcSwitchVal(Schedule_T *dp);
 uint8_t            Is_Pattern(uint8_t Type);
 void               Print_Comment(uint8_t Type);
 void               Int_Update(uint32_t Time);
 void               Inc_cp(uint8_t Type);
 uint8_t            Find_Next_Priv_InpChannel(uint8_t Channel, int8_t Direction);


 // Test_buttons
 void               Setup_Test_Buttons();
 void               Set_one_Test_Outp(uint8_t Nr, CRGB* &p);
 void               Set_Test_Outp(uint8_t &Ch);
 void               Speed_Test();
 void               Proc_Key_Test(uint8_t key);
 void               Proc_Test_Buttons();

 // Proc functions called in Update
 void               Proc_Const();
 void               Proc_AnalogPattern(uint8_t TimeCnt, bool AnalogMode);
 void               Proc_House();
 void               Proc_Fire();
 #ifdef _NEW_ROOM_COL
  void              Proc_Set_ColTab();
 #endif
 void               Proc_Set_TV_Tab();                                                                        // 09.01.20:
 void               Proc_Set_Def_Neon();                                                                      // 12.01.20:
 void               Proc_Logic();
 void               Proc_New_HSV_Group();
 void               Proc_New_Local_Var();                                                                     // 07.11.18:
 void               Proc_Use_GlobalVar();
 void               Proc_InCh_to_TmpVar();
 void               Proc_Bin_InCh_to_TmpVar();                                                                // 18.01.19:
 void               Proc_Random();
 void               Proc_RandMux();
 void               Proc_Welding();
 void               Proc_CopyLED();
 void               Proc_Schedule();
 void               Proc_Counter();

 void               IncCP_House();
 void               IncCP_Const()         { cp += 5; }
 void               IncCP_Fire()          { cp += 4; }
 #ifdef _NEW_ROOM_COL
   void             IncCP_Set_ColTab()    { cp += ROOM_COL_CNT*3; }
   bool             Cmp_Room_Col(CRGB *lp, uint8_t ColorNr);
   uint8_t          Get_Room_Col1(uint8_t ColorNr, uint8_t Channel);
   void             Copy_Room_Col(CRGB *Dst, uint8_t ColorNr);
   void             Copy_Single_Room_Col(CRGB *Dst, uint8_t Channel, uint8_t ColorNr);                        // 06.09.19:
 #endif
 void               IncCP_Set_TV_Tab();                                                                       // 10.01.20:
 void               IncCP_Set_Def_Neon();                                                                     // 12.01.20:
 void               IncCP_Pattern(uint8_t TimeCnt);
 void               IncCP_Logic();
 void               IncCP_New_HSV_Group() { }
 void               IncCP_New_Local_Var() { }                                                                 // 07.11.18:
 void               IncCP_Use_GlobalVar() { cp++; }
 void               IncCP_InCh_to_TmpVar(){ cp+=2; }                                                          // 25.11.18:
 void               IncCP_Random()        { cp += EP_RANDOM_INCREMENT;   }
 void               IncCP_RandMux()       { cp += EP_RANDMUX_INCREMENT;  }
 void               IncCP_Welding()       { cp += EP_WELDING_INCREMENT;  }
 void               IncCP_CopyLED()       { cp += EP_COPYLED_INCREMENT;  }
 void               IncCP_Schedule()      { cp += EP_SCHEDULE_INCREMENT; }
 void               IncCP_Counter();

 uint8_t            InpChannel_used(uint8_t Channel);
 bool               Inp_is_Used_in_Logic(uint8_t Channel);



#ifdef _NEW_ROOM_COL
   const uint8_t     *Room_ColP;
#endif

#if _USE_SET_TVTAB                                                                                            // 09.01.20:
   const uint8_t     *TV_Dat_p[_TV_CHANNELS];
#endif

}; // class MobaLedLib_C



#endif // _MOBALEDLIB_H_


